@@ -22,6 +22,7 @@ class AbortInstanceStartException(SchedulerException):
     pass
 
 
+# Database
 class DatabaseException(Exception):
     pass
 
@@ -38,6 +39,10 @@ class ReadException(DatabaseException):
     pass
 
 
+class BadMetadataPacket(DatabaseException):
+    pass
+
+
 class VirtException(Exception):
     pass
 
