@@ -40,9 +40,6 @@ class SFPyLogrus(logging.Logger, PyLogrusBase):
             raise Exception('Bad object - no unique_label() function: %s' % e)
         return SFCustomAdapter(self, {label: value})
 
-    def withLabel(self, label, value):
-        return SFCustomAdapter(self, {label: value})
-
     #
     # Use labelled convenience methods when ID is a string (not object)
     # Note: the helper method still handles objects
@@ -64,7 +61,7 @@ class SFPyLogrus(logging.Logger, PyLogrusBase):
 
     def withImage(self, inst):
         if not isinstance(inst, str):
-            inst = inst.hashed_image_url
+            inst = inst.unique_ref
         return SFCustomAdapter(self, {'image': inst})
 
 
@@ -136,9 +133,6 @@ class SFCustomAdapter(logging.LoggerAdapter, PyLogrusBase):
             extra.update({label: value})
         return SFCustomAdapter(self._logger, extra, self._prefix)
 
-    def withLabel(self, label, value):
-        return SFCustomAdapter(self, {label: value})
-
     #
     # Use labelled convenience methods when ID is a string (not object)
     # Note: the helper method still handles objects
@@ -167,7 +161,7 @@ class SFCustomAdapter(logging.LoggerAdapter, PyLogrusBase):
     def withImage(self, inst):
         extra = copy.deepcopy(self._extra)
         if not isinstance(inst, str):
-            inst = inst.hashed_image_url
+            inst = inst.unique_ref
         extra.update({'image': inst})
         return SFCustomAdapter(self._logger, extra, self._prefix)
 
@@ -179,6 +173,7 @@ def setup(name):
     logging.root.setLevel(logging.DEBUG)
     log = logging.getLogger(name)
 
+    handler = None
     if log.hasHandlers():
         # The parent logger might have the handler, not this lower logger
         if len(log.handlers) > 0:
