@@ -2,13 +2,13 @@ import base64
 import bcrypt
 import json
 import mock
-import testtools
 
 
 from shakenfist import config
 from shakenfist.external_api import app as external_api
 from shakenfist import ipmanager
 from shakenfist import net
+from shakenfist.tests import test_shakenfist
 
 
 class FakeResponse(object):
@@ -43,7 +43,7 @@ def _clean_traceback(resp):
     return resp
 
 
-class AuthTestCase(testtools.TestCase):
+class AuthTestCase(test_shakenfist.ShakenFistTestCase):
     def setUp(self):
         super(AuthTestCase, self).setUp()
 
@@ -172,7 +172,7 @@ class AuthTestCase(testtools.TestCase):
         self.assertEqual(401, resp.status_code)
 
 
-class ExternalApiTestCase(testtools.TestCase):
+class ExternalApiTestCase(test_shakenfist.ShakenFistTestCase):
     def setUp(self):
         super(ExternalApiTestCase, self).setUp()
 
