@@ -7,6 +7,7 @@ from pbr.version import VersionInfo
 import random
 import re
 import requests
+import secrets
 import string
 import sys
 import time
@@ -63,7 +64,7 @@ class RecordedOperation():
 
 
 def is_network_node():
-    """Test if this node is the network node"""
+    """Test if this node is the network node."""
     return config.parsed.get('NODE_IP') == config.parsed.get('NETWORK_NODE_IP')
 
 
@@ -187,14 +188,15 @@ def extract_power_state(libvirt, domain):
 
 
 def get_api_token(base_url, namespace='system'):
-    with db.get_lock('namespace', None, namespace):
+    with db.get_lock('namespace', None, namespace,
+                     op='Get API token'):
         auth_url = base_url + '/auth'
         LOG.info('Fetching %s auth token from %s' % (namespace, auth_url))
         ns = db.get_namespace(namespace)
         if 'service_key' in ns:
             key = ns['service_key']
         else:
-            key = ''.join(random.choice(string.ascii_lowercase)
+            key = ''.join(secrets.choice(string.ascii_lowercase)
                           for i in range(50))
             ns['service_key'] = key
             db.persist_namespace(namespace, ns)
