@@ -163,8 +163,8 @@ class Instance(object):
             modified_disks = []
             for disk in self.db_entry['block_devices']['devices']:
                 if disk.get('base'):
-                    img = images.Image(disk['base'])
-                    hashed_image_path = img.image_path()
+                    img = images.Image.from_url(disk['base'])
+                    hashed_image_path = img.version_image_path()
 
                     with util.RecordedOperation('detect cdrom images', self):
                         try:
@@ -191,27 +191,33 @@ class Instance(object):
                         disk['device'] = 'hd%s' % disk['device'][-1]
                         disk['bus'] = 'ide'
                     else:
-                        with util.RecordedOperation('resize image', self):
-                            resized_image_path = images.resize(
-                                [lock], hashed_image_path, disk['size'])
-
                         if config.parsed.get('DISK_FORMAT') == 'qcow':
                             with util.RecordedOperation('create copy on write layer', self):
-                                images.create_cow(
-                                    [lock], resized_image_path, disk['path'])
+                                images.create_cow([lock], hashed_image_path,
+                                                  disk['path'], disk['size'])
 
                             # Record the backing store for modern libvirts
-                            disk['backing'] = ('<backingStore type=\'file\'>\n'
-                                               '        <format type=\'qcow2\'/>\n'
-                                               '        <source file=\'%s\'/>\n'
-                                               '      </backingStore>' % resized_image_path)
+                            disk['backing'] = (
+                                '<backingStore type=\'file\'>\n'
+                                '        <format type=\'qcow2\'/>\n'
+                                '        <source file=\'%s\'/>\n'
+                                '      </backingStore>\n'
+                                % (hashed_image_path))
 
                         elif config.parsed.get('DISK_FORMAT') == 'qcow_flat':
+                            with util.RecordedOperation('resize image', self):
+                                resized_image_path = img.resize(
+                                    [lock], disk['size'])
+
                             with util.RecordedOperation('create flat layer', self):
                                 images.create_flat(
                                     [lock], resized_image_path, disk['path'])
 
                         elif config.parsed.get('DISK_FORMAT') == 'flat':
+                            with util.RecordedOperation('resize image', self):
+                                resized_image_path = img.resize(
+                                    [lock], disk['size'])
+
                             with util.RecordedOperation('create raw disk', self):
                                 images.create_raw(
                                     [lock], resized_image_path, disk['path'])
@@ -274,7 +280,7 @@ class Instance(object):
             for ni in db.get_instance_interfaces(self.db_entry['uuid']):
                 db.update_network_interface_state(ni['uuid'], 'deleted')
                 with db.get_lock('ipmanager', None, ni['network_uuid'],
-                                 ttl=120):
+                                 ttl=120, op='Instance delete'):
                     ipm = db.get_ipmanager(ni['network_uuid'])
                     ipm.release(ni['ipv4'])
                     db.persist_ipmanager(ni['network_uuid'], ipm.save())
@@ -282,6 +288,16 @@ class Instance(object):
         db.free_console_port(self.db_entry['console_port'])
         db.free_console_port(self.db_entry['vdi_port'])
 
+    def allocate_instance_ports(self):
+        uuid = self.db_entry['uuid']
+        self.db_entry['console_port'] = db.allocate_console_port(uuid)
+        self.db_entry['vdi_port'] = db.allocate_console_port(uuid)
+
+        # TODO(andy): When class is modified to model Image class this
+        # repetitive write will be moved to a single persist() function.
+        db.persist_console_ports(
+            uuid, self.db_entry['console_port'], self.db_entry['vdi_port'])
+
     def _make_config_drive(self, disk_path):
         """Create a config drive"""
 
